// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "UIComponent.hpp"

using namespace Dawn;

UIComponent::UIComponent(UICanvas &canvas) :
  canvas(canvas)
{
}

void UIComponent::updatePositions() {
  // X Alignment
  if(this->alignX == UI_COMPONENT_ALIGN_STRETCH) {
    if(this->parent == nullptr) {
      this->width = this->canvas.getWidth();
    } else {
      this->width = this->parent->getWidth();
    }
    this->relativeX = this->alignment[0];
    this->width -= (this->alignment[0] + this->alignment[2]);
  } else {
    this->relativeX = this->alignment[0];
    this->width = this->alignment[2];
  }

  // Y Alignment
  if(this->alignY == UI_COMPONENT_ALIGN_STRETCH) {
    if(this->parent == nullptr) {
      this->height = this->canvas.getHeight();
    } else {
      this->height = this->parent->getHeight();
    }
    this->relativeY = this->alignment[1];
    this->height -= (this->alignment[1] + this->alignment[3]);
  } else {
    this->relativeY = this->alignment[1];
    this->height = this->alignment[3];
  }

  // Update children
  auto it = this->children.begin();
  while(it != this->children.end()) {
    (*it)->updatePositions();
    ++it;
  }
}

float_t UIComponent::getWidth() {
  return this->width;
}

float_t UIComponent::getHeight() {
  return this->height;
}

float_t UIComponent::getRelativeX() {
  return this->relativeX;
}

float_t UIComponent::getRelativeY() {
  return this->relativeY;
}

void UIComponent::setTransform(
  UIComponentAlign xAlign,
  UIComponentAlign yAlign,
  glm::vec4 alignment,
  float_t z
) {
  this->alignX = xAlign;
  this->alignY = yAlign;
  this->alignment = alignment;
  this->z = z;
  this->updatePositions();
}

void UIComponent::draw(UIShader &uiShader, glm::mat4 parentTransform) {
  // Calculate self transform matrix
  glm::mat4 selfTransform = parentTransform * glm::translate(
    glm::mat4(1.0f), glm::vec3(this->relativeX, this->relativeY, this->z)
  );

  // Draw Self
  this->drawSelf(uiShader, selfTransform);

  // Render children
  auto it = this->children.begin();
  while(it != this->children.end()) {
    (*it)->draw(uiShader, selfTransform);
    ++it;
  }
}

void UIComponent::addChild(std::shared_ptr<UIComponent> child) {
  if(child->parent == this) return;
  if(child->parent != nullptr) child->parent->removeChild(child);
  this->children.push_back(child);
  child->parent = this;
}

void UIComponent::removeChild(std::shared_ptr<UIComponent> child) {
  if(child->parent != this) throw "Invalid child";
  auto it = this->children.begin();
  while(it != this->children.end()) {
    if(it->get() == child.get()) {
      this->children.erase(it);
      break;
    }
    ++it;
  }
}

UIComponent::~UIComponent() {
  
}